@@ -1,13 +1,13 @@
 <?php
 /**
- * $Id: messagehandler.php,v 1.25.2.5 2004/09/23 23:24:22 meebey Exp $
- * $Revision: 1.25.2.5 $
- * $Author: meebey $
- * $Date: 2004/09/23 23:24:22 $
+ * $Id: messagehandler.php 241017 2007-08-10 09:22:02Z amir $
+ * $Revision: 241017 $
+ * $Author: amir $
+ * $Date: 2007-08-10 18:52:02 +0930 (Fri, 10 Aug 2007) $
  *
- * Copyright (c) 2002-2003 Mirco "MEEBEY" Bauer <mail@meebey.net> <http://www.meebey.net>
+ * Copyright (c) 2002-2004 Mirco Bauer <meebey@meebey.net> <http://www.meebey.net>
  * 
- * Full LGPL License: <http://www.meebey.net/lgpl.txt>
+ * Full LGPL License: <http://www.gnu.org/licenses/lgpl.txt>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -35,6 +35,7 @@ class Net_SmartIRC_messagehandler extends Net_SmartIRC_irccommands
     function _event_error(&$ircdata)
     {
         if ($this->_autoretry == true) {
+            $this->_delayReconnect();
             $this->reconnect();
         } else {
             $this->disconnect(true);
@@ -48,11 +49,18 @@ class Net_SmartIRC_messagehandler extends Net_SmartIRC_irccommands
                 $this->log(SMARTIRC_DEBUG_CHANNELSYNCING, 'DEBUG_CHANNELSYNCING: joining channel: '.$ircdata->channel, __FILE__, __LINE__);
                 $channel = &new Net_SmartIRC_channel();
                 $channel->name = $ircdata->channel;
+                $microint = $this->_microint();
+                $channel->synctime_start = $microint;
+                $this->log(SMARTIRC_DEBUG_CHANNELSYNCING, 'DEBUG_CHANNELSYNCING: synctime_start for '.$ircdata->channel.' set to: '.$microint, __FILE__, __LINE__);
                 $this->_channels[strtolower($channel->name)] = &$channel;
                 
-                $this->who($channel->name);
+                // the class will get his own who data from the whole who channel list
                 $this->mode($channel->name);
+                $this->who($channel->name);
                 $this->ban($channel->name);
+            } else {
+                // the class didn't join but someone else, lets get his who data
+                $this->who($ircdata->nick);
             }
             
             $this->log(SMARTIRC_DEBUG_CHANNELSYNCING, 'DEBUG_CHANNELSYNCING: '.$ircdata->nick.' joins channel: '.$ircdata->channel, __FILE__, __LINE__);
@@ -63,7 +71,6 @@ class Net_SmartIRC_messagehandler extends Net_SmartIRC_irccommands
             $user->host = $ircdata->host;
             
             $this->_adduser($channel, $user);
-            $this->who($user->nick);
         }
     }
     
@@ -95,10 +102,11 @@ class Net_SmartIRC_messagehandler extends Net_SmartIRC_irccommands
             $lowerednewnick = strtolower($newnick);
             $lowerednick = strtolower($ircdata->nick);
             
-            foreach ($this->_channels as $channelkey => $channelvalue) {
+            $channelkeys = array_keys($this->_channels);
+            foreach ($channelkeys as $channelkey) {
                 // loop through all channels
                 $channel = &$this->_channels[$channelkey];
-                foreach ($channel->users as $userkey => $uservalue) {
+                foreach ($channel->users as $uservalue) {
                     // loop through all user in this channel
                     
                     if ($ircdata->nick == $uservalue->nick) {
@@ -142,7 +150,6 @@ class Net_SmartIRC_messagehandler extends Net_SmartIRC_irccommands
             
             $add = false;
             $remove = false;
-            $channelmode = '';
             $modelength = strlen($mode);
             for ($i = 0; $i < $modelength; $i++) {
                 switch($mode[$i]) {
@@ -194,6 +201,17 @@ class Net_SmartIRC_messagehandler extends Net_SmartIRC_irccommands
                             $channel->key = '';
                         }
                     break;
+                    case 'l':
+                        if ($add) {
+                            $limit = array_shift($parameters);
+                            $this->log(SMARTIRC_DEBUG_CHANNELSYNCING, 'DEBUG_CHANNELSYNCING: stored user limit for: '.$channel->name, __FILE__, __LINE__);
+                            $channel->user_limit = $limit;
+                        }
+                        if ($remove) {
+                            $this->log(SMARTIRC_DEBUG_CHANNELSYNCING, 'DEBUG_CHANNELSYNCING: removed user limit for: '.$channel->name, __FILE__, __LINE__);
+                            $channel->user_limit = false;
+                        }
+                    break;
                     default:
                         // channel modes
                         if ($mode[$i] == 'b') {
@@ -230,18 +248,20 @@ class Net_SmartIRC_messagehandler extends Net_SmartIRC_irccommands
     
     function _event_privmsg(&$ircdata)
     {
-        if ($ircdata->type == SMARTIRC_TYPE_CTCP) {
+        if ($ircdata->type & SMARTIRC_TYPE_CTCP_REQUEST) {
             // substr must be 1,4 because of \001 in CTCP messages
             if (substr($ircdata->message, 1, 4) == 'PING') {
-                $this->message(SMARTIRC_TYPE_CTCP, $ircdata->nick, 'PING '.substr($ircdata->message, 5, -1));
+                $this->message(SMARTIRC_TYPE_CTCP_REPLY, $ircdata->nick, 'PING '.substr($ircdata->message, 5, -1));
             } elseif (substr($ircdata->message, 1, 7) == 'VERSION') {
                 if (!empty($this->_ctcpversion)) {
-                    $versionstring = $this->_ctcpversion.' | using '.SMARTIRC_VERSIONSTRING;
+                    $versionstring = $this->_ctcpversion;
                 } else {
                     $versionstring = SMARTIRC_VERSIONSTRING;
                 }
                 
-                $this->message(SMARTIRC_TYPE_CTCP, $ircdata->nick, 'VERSION '.$versionstring);
+                $this->message(SMARTIRC_TYPE_CTCP_REPLY, $ircdata->nick, 'VERSION '.$versionstring);
+            } elseif (substr($ircdata->message, 1, 10) == 'CLIENTINFO') {
+                $this->message(SMARTIRC_TYPE_CTCP_REPLY, $ircdata->nick, 'CLIENTINFO PING VERSION CLIENTINFO');
             }
         }
     }
@@ -312,7 +332,7 @@ class Net_SmartIRC_messagehandler extends Net_SmartIRC_irccommands
                 if (!$this->isJoined($ircdata->channel, $nick)) {
                     return;
                 }
-                                
+                
                 $channel = &$this->_channels[strtolower($ircdata->channel)];
                 
                 $user = &new Net_SmartIRC_channeluser();
@@ -375,6 +395,12 @@ class Net_SmartIRC_messagehandler extends Net_SmartIRC_irccommands
                         $user->voice = true;
                         $user->nick = substr($userarray[$i], 1);
                     break;
+                    // RFC violating IRC servers might break us
+                    case '~':
+                    case '&':
+                    case '%':
+                        $user->nick = substr($userarray[$i], 1);
+                    break;
                     default:
                         $user->nick = $userarray[$i];
                 }
@@ -393,6 +419,22 @@ class Net_SmartIRC_messagehandler extends Net_SmartIRC_irccommands
         }
     }
     
+    function _event_rpl_endofbanlist(&$ircdata)
+    {
+        if ($this->_channelsyncing == true && $this->isJoined($ircdata->channel)) {
+            $channel = &$this->getChannel($ircdata->channel);
+            if ($channel->synctime_stop == 0) {
+                // we received end of banlist and the stop timestamp is not set yet
+                $microint = $this->_microint();
+                $channel->synctime_stop = $microint;
+                $this->log(SMARTIRC_DEBUG_CHANNELSYNCING, 'DEBUG_CHANNELSYNCING: synctime_stop for '.$ircdata->channel.' set to: '.$microint, __FILE__, __LINE__);
+                
+                $channel->synctime = ((float)$channel->synctime_stop - (float)$channel->synctime_start);
+                $this->log(SMARTIRC_DEBUG_CHANNELSYNCING, 'DEBUG_CHANNELSYNCING: synced channel '.$ircdata->channel.' in '.round($channel->synctime, 2).' secs', __FILE__, __LINE__);
+            }
+        }
+    }
+    
     function _event_rpl_topic(&$ircdata)
     {
         if ($this->_channelsyncing == true) {
@@ -408,4 +450,4 @@ class Net_SmartIRC_messagehandler extends Net_SmartIRC_irccommands
         $this->_nicknameinuse();
     }
 }
-?>
\ No newline at end of file
+?>
